@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -29,10 +29,12 @@ extern "C" {
 #endif
 
 #define CONNMAN_SERVICE			"org.moblin.connman"
+#define CONNMAN_PATH			"/org/moblin/connman"
 
 #define CONNMAN_DEBUG_INTERFACE		CONNMAN_SERVICE ".Debug"
 #define CONNMAN_ERROR_INTERFACE		CONNMAN_SERVICE ".Error"
 #define CONNMAN_AGENT_INTERFACE		CONNMAN_SERVICE ".Agent"
+#define CONNMAN_COUNTER_INTERFACE	CONNMAN_SERVICE ".Counter"
 
 #define CONNMAN_MANAGER_INTERFACE	CONNMAN_SERVICE ".Manager"
 #define CONNMAN_MANAGER_PATH		"/"
@@ -43,6 +45,10 @@ extern "C" {
 #define CONNMAN_DEVICE_INTERFACE	CONNMAN_SERVICE ".Device"
 #define CONNMAN_NETWORK_INTERFACE	CONNMAN_SERVICE ".Network"
 #define CONNMAN_PROVIDER_INTERFACE	CONNMAN_SERVICE ".Provider"
+#define CONNMAN_TECHNOLOGY_INTERFACE	CONNMAN_SERVICE ".Technology"
+
+#define CONNMAN_PRIVILEGE_MODIFY	1
+#define CONNMAN_PRIVILEGE_SECRET	2
 
 typedef void (* connman_dbus_append_cb_t) (DBusMessageIter *iter,
 							void *user_data);
