@@ -205,7 +205,7 @@ int __connman_device_enable(struct connman_device *device)
 {
 	int err;
 
-	DBG("device %p", device);
+	DBG("device %p %d", device, device->blocked);
 
 	if (!device->driver || !device->driver->enable)
 		return -EOPNOTSUPP;
@@ -559,9 +559,6 @@ static int setup_device(struct connman_device *device)
 		break;
 	}
 
-	if (__connman_udev_get_blocked(device->phyindex) == TRUE)
-		return 0;
-
 	if (device->offlinemode == FALSE &&
 				device->powered_persistent == TRUE)
 		__connman_device_enable(device);
@@ -589,6 +586,8 @@ static void probe_driver(struct connman_element *element, gpointer user_data)
 
 	element->device->driver = driver;
 
+	__connman_element_set_driver(element);
+
 	setup_device(element->device);
 }
 
@@ -745,6 +744,7 @@ struct connman_device *connman_device_create(const char *node,
 {
 	struct connman_device *device;
 	const char *str;
+	enum connman_service_type service_type;
 
 	DBG("node %s type %d", node, type);
 
@@ -777,6 +777,9 @@ struct connman_device *connman_device_create(const char *node,
 
 	device->phyindex = -1;
 
+	service_type = __connman_device_get_service_type(device);
+	device->blocked = __connman_technology_get_blocked(service_type);
+
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
@@ -1653,7 +1656,7 @@ static int device_load(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		val = g_key_file_get_integer(keyfile, identifier,
 						"ScanInterval", &error);
-		if (error == NULL && val > 0)
+		if (error == NULL)
 			device->scan_interval = val;
 		g_clear_error(&error);
 		break;
@@ -1691,8 +1694,7 @@ static int device_save(struct connman_device *device)
 		break;
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		if (device->scan_interval > 0)
-			g_key_file_set_integer(keyfile, identifier,
+		g_key_file_set_integer(keyfile, identifier,
 					"ScanInterval", device->scan_interval);
 		break;
 	}
