@@ -36,8 +36,8 @@ static DBusConnection *connection;
 
 static GNode *element_root = NULL;
 static GSList *driver_list = NULL;
-static gchar *device_filter = NULL;
-static gchar *nodevice_filter = NULL;
+static gchar **device_filter = NULL;
+static gchar **nodevice_filter = NULL;
 
 static gboolean started = FALSE;
 
@@ -176,7 +176,7 @@ void __connman_element_list(struct connman_element *element,
 						append_path, &filter);
 }
 
-static struct connman_network *__connman_element_get_network(struct connman_element *element)
+static struct connman_network *get_network(struct connman_element *element)
 {
 	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
 						element->network != NULL)
@@ -185,7 +185,7 @@ static struct connman_network *__connman_element_get_network(struct connman_elem
 	if (element->parent == NULL)
 		return NULL;
 
-	return __connman_element_get_network(element->parent);
+	return get_network(element->parent);
 }
 
 struct connman_service *__connman_element_get_service(struct connman_element *element)
@@ -211,7 +211,7 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	case CONNMAN_DEVICE_TYPE_WIMAX:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 	case CONNMAN_DEVICE_TYPE_CELLULAR:
-		network = __connman_element_get_network(element);
+		network = get_network(element);
 		if (network == NULL)
 			return NULL;
 		service = __connman_service_lookup_from_network(network);
@@ -273,7 +273,7 @@ static gboolean find_device(GNode *node, gpointer user_data)
 	if (element->device == NULL)
 		return FALSE;
 
-	if (data->type != connman_device_get_type(element->device))
+	if (data->type != __connman_device_get_service_type(element->device))
 		return FALSE;
 
 	data->device = element->device;
@@ -777,6 +777,12 @@ int connman_element_get_value(struct connman_element *element,
 								id, value);
 		*((char **) value) = element->ipv4.address;
 		break;
+	case CONNMAN_PROPERTY_ID_IPV4_PEER:
+		if (element->ipv4.peer == NULL)
+			return connman_element_get_value(element->parent,
+								id, value);
+		*((char **) value) = element->ipv4.peer;
+		break;
 	case CONNMAN_PROPERTY_ID_IPV4_NETMASK:
 		if (element->ipv4.netmask == NULL)
 			return connman_element_get_value(element->parent,
@@ -1063,21 +1069,27 @@ static void register_element(gpointer data, gpointer user_data)
 
 gboolean __connman_element_device_isfiltered(const char *devname)
 {
+	char **pattern;
+
 	if (device_filter == NULL)
 		goto nodevice;
 
-	if (g_pattern_match_simple(device_filter, devname) == FALSE) {
-		DBG("ignoring device %s (match)", devname);
-		return TRUE;
+	for (pattern = device_filter; *pattern; pattern++) {
+		if (g_pattern_match_simple(*pattern, devname) == FALSE) {
+			DBG("ignoring device %s (match)", devname);
+			return TRUE;
+		}
 	}
 
 nodevice:
 	if (nodevice_filter == NULL)
 		return FALSE;
 
-	if (g_pattern_match_simple(nodevice_filter, devname) == TRUE) {
-		DBG("ignoring device %s (no match)", devname);
-		return TRUE;
+	for (pattern = nodevice_filter; *pattern; pattern++) {
+		if (g_pattern_match_simple(*pattern, devname) == TRUE) {
+			DBG("ignoring device %s (no match)", devname);
+			return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -1138,8 +1150,7 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 	if (element == root)
 		return FALSE;
 
-	if (node != NULL)
-		g_node_unlink(node);
+	g_node_unlink(node);
 
 	if (element->driver) {
 		if (element->driver->remove)
@@ -1148,8 +1159,7 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		element->driver = NULL;
 	}
 
-	if (node != NULL)
-		g_node_destroy(node);
+	g_node_destroy(node);
 
 	connman_element_unref(element);
 
@@ -1176,8 +1186,7 @@ static gboolean remove_element_type(GNode *node, gpointer user_data)
 	if(element->type != type)
 		return FALSE;
 
-	if (node != NULL)
-		g_node_unlink(node);
+	g_node_unlink(node);
 
 	if (element->driver) {
 		if (element->driver->remove)
@@ -1186,8 +1195,7 @@ static gboolean remove_element_type(GNode *node, gpointer user_data)
 		element->driver = NULL;
 	}
 
-	if (node != NULL)
-		g_node_destroy(node);
+	g_node_destroy(node);
 
 	connman_element_unref(element);
 
@@ -1320,6 +1328,28 @@ void connman_element_set_error(struct connman_element *element,
 	__connman_service_indicate_error(service, convert_error(error));
 }
 
+void __connman_element_set_driver(struct connman_element *element)
+{
+	GSList *list;
+
+	DBG("element %p name %s driver %p", element, element->name,
+						element->driver);
+
+	if (element->driver)
+		return;
+
+	for (list = driver_list; list; list = list->next) {
+		struct connman_driver *driver = list->data;
+
+		if (match_driver(element, driver) == FALSE)
+			continue;
+
+		element->driver = driver;
+
+		break;
+	}
+}
+
 int __connman_element_init(const char *device, const char *nodevice)
 {
 	struct connman_element *element;
@@ -1330,8 +1360,11 @@ int __connman_element_init(const char *device, const char *nodevice)
 	if (connection == NULL)
 		return -EIO;
 
-	device_filter = g_strdup(device);
-	nodevice_filter = g_strdup(nodevice);
+	if (device)
+		device_filter = g_strsplit(device, ",", -1);
+
+	if (nodevice)
+		nodevice_filter = g_strsplit(nodevice, ",", -1);
 
 	element = connman_element_create("root");
 
@@ -1380,7 +1413,6 @@ void __connman_element_start(void)
 	started = TRUE;
 
 	__connman_rtnl_start();
-	__connman_udev_start();
 
 	__connman_connection_init();
 	__connman_ipv4_init();
@@ -1451,8 +1483,8 @@ void __connman_element_cleanup(void)
 	g_node_destroy(element_root);
 	element_root = NULL;
 
-	g_free(nodevice_filter);
-	g_free(device_filter);
+	g_strfreev(nodevice_filter);
+	g_strfreev(device_filter);
 
 	if (connection == NULL)
 		return;
