@@ -106,10 +106,6 @@ static DBusMessage *get_properties(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_PUBLIC) < 0)
-		return __connman_error_permission_denied(msg);
-
 	reply = dbus_message_new_method_return(msg);
 	if (reply == NULL)
 		return NULL;
@@ -168,9 +164,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	}
 
 
-	if (network->wifi.passphrase != NULL &&
-			__connman_security_check_privilege(msg,
-				CONNMAN_SECURITY_PRIVILEGE_SECRET) == 0)
+	if (network->wifi.passphrase != NULL)
 		connman_dbus_dict_append_basic(&dict, "WiFi.Passphrase",
 				DBUS_TYPE_STRING, &network->wifi.passphrase);
 
@@ -272,8 +266,18 @@ static gint compare_priority(gconstpointer a, gconstpointer b)
  */
 int connman_network_driver_register(struct connman_network_driver *driver)
 {
+	GSList *list;
+
 	DBG("driver %p name %s", driver, driver->name);
 
+	for (list = driver_list; list; list = list->next) {
+		struct connman_network_driver *tmp = list->data;
+
+		if (tmp->type == driver->type)
+			return -EALREADY;
+
+	}
+
 	driver_list = g_slist_insert_sorted(driver_list, driver,
 							compare_priority);
 
@@ -314,6 +318,8 @@ static void network_destruct(struct connman_element *element)
 		connman_ipconfig_unref(network->ipconfig);
 		network->ipconfig = NULL;
 	}
+
+	network->device = NULL;
 }
 
 /**
@@ -444,19 +450,31 @@ const char *connman_network_get_path(struct connman_network *network)
  */
 void connman_network_set_index(struct connman_network *network, int index)
 {
-	if (network->element.index < 0) {
-		struct connman_service *service;
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
 
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		goto done;
+
+	ipconfig = __connman_service_get_ipconfig(service);
+
+	if (network->element.index < 0 && ipconfig == NULL)
 		/*
 		 * This is needed for plugins that havent set their ipconfig
 		 * layer yet, due to not being able to get a network index
 		 * prior to creating a service.
 		 */
-		service = __connman_service_lookup_from_network(network);
-		if (service != NULL)
-			__connman_service_create_ipconfig(service, index);
+		__connman_service_create_ipconfig(service, index);
+	else {
+		/* If index changed, the index of ipconfig must be reset. */
+		if (ipconfig == NULL)
+			goto done;
+
+		__connman_ipconfig_set_index(ipconfig, index);
 	}
 
+done:
 	network->element.index = index;
 }
 
@@ -520,21 +538,24 @@ void connman_network_set_group(struct connman_network *network,
 	}
 
 	if (g_strcmp0(network->group, group) == 0) {
-		if (group != NULL)
+		if (group != NULL && network->registered)
 			__connman_profile_update_network(network);
 		return;
 	}
 
 	if (network->group != NULL) {
-		__connman_profile_remove_network(network);
+		if (network->registered)
+			__connman_profile_remove_network(network);
 
 		g_free(network->group);
 	}
 
 	network->group = g_strdup(group);
 
-	if (network->group != NULL)
-		__connman_profile_add_network(network);
+	if (network->group != NULL) {
+		if (network->registered)
+			__connman_profile_add_network(network);
+	}
 }
 
 /**
@@ -553,7 +574,7 @@ const char *__connman_network_get_ident(struct connman_network *network)
 	if (network->device == NULL)
 		return NULL;
 
-	return __connman_device_get_ident(network->device);
+	return connman_device_get_ident(network->device);
 }
 
 connman_bool_t __connman_network_get_weakness(struct connman_network *network)
@@ -727,10 +748,51 @@ static void set_configuration(struct connman_network *network)
 					CONNMAN_SERVICE_STATE_CONFIGURATION);
 }
 
+static int set_connected_fixed(struct connman_network *network)
+{
+	struct connman_service *service;
+	struct connman_element *parent, *element;
+
+	DBG("");
+
+	service = __connman_service_lookup_from_network(network);
+
+	parent = connman_network_get_element(network);
+
+	set_configuration(network);
+
+	if (parent->ipv4.address == NULL)
+		return -EINVAL;
+
+	if (parent->ipv4.netmask == NULL)
+		return -EINVAL;
+
+	element = connman_element_create(NULL);
+	if (element == NULL) {
+		connman_error("Can not create connman_element");
+		return -ENOMEM;
+	}
+
+	element->type = CONNMAN_ELEMENT_TYPE_IPV4;
+	element->index = parent->index;
+
+	if (connman_element_register(element, parent) < 0) {
+		connman_error("Can not register connman_element");
+		return -EINVAL;
+	}
+
+	network->connecting = FALSE;
+
+	connman_network_set_associating(network, FALSE);
+
+	return 0;
+}
+
 static void set_connected_manual(struct connman_network *network)
 {
 	struct connman_service *service;
 	struct connman_ipconfig *ipconfig;
+	const char *nameserver = NULL;
 	int err;
 
 	DBG("network %p", network);
@@ -748,6 +810,11 @@ static void set_connected_manual(struct connman_network *network)
 		return;
 	}
 
+	connman_element_get_value(&network->element,
+			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
+	if (nameserver != NULL)
+		__connman_service_append_nameserver(service, nameserver);
+
 	__connman_ipconfig_set_gateway(ipconfig, &network->element);
 
 	network->connecting = FALSE;
@@ -785,6 +852,31 @@ static int set_connected_dhcp(struct connman_network *network)
 	return 0;
 }
 
+static int manual_ipv6_set(struct connman_network *network,
+				struct connman_ipconfig *ipconfig_ipv6)
+{
+	struct connman_service *service;
+	int err;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return -EINVAL;
+
+	err = __connman_ipconfig_set_address(ipconfig_ipv6);
+	if (err < 0) {
+		connman_network_set_error(network,
+			CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL);
+		return err;
+	}
+
+	/*
+	 * READY state will be indicated by IPV4 setting
+	 * gateway will be set by IPV4 setting
+	 */
+
+	return 0;
+}
+
 static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
@@ -801,12 +893,41 @@ static gboolean set_connected(gpointer user_data)
 	DBG("method %d", method);
 
 	if (network->connected == TRUE) {
+		enum connman_ipconfig_method ipv6_method;
+		struct connman_ipconfig *ipv6config;
+		int ret;
+
+		ipv6config = connman_ipconfig_get_ipv6config(ipconfig);
+		ipv6_method = __connman_ipconfig_get_method(ipv6config);
+		switch (ipv6_method) {
+		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+		case CONNMAN_IPCONFIG_METHOD_OFF:
+			break;
+		case CONNMAN_IPCONFIG_METHOD_FIXED:
+		case CONNMAN_IPCONFIG_METHOD_MANUAL:
+			ret = manual_ipv6_set(network, ipv6config);
+			if (ret != 0) {
+				connman_network_set_error(network,
+					CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
+				return FALSE;
+			}
+			break;
+		case CONNMAN_IPCONFIG_METHOD_DHCP:
+			break;
+		}
+
 		switch (method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
 			return FALSE;
-		case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
+			if (set_connected_fixed(network) < 0) {
+				connman_network_set_error(network,
+					CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
+				return FALSE;
+			}
+			return TRUE;
+		case CONNMAN_IPCONFIG_METHOD_MANUAL:
 			set_connected_manual(network);
 			return TRUE;
 		case CONNMAN_IPCONFIG_METHOD_DHCP:
@@ -908,6 +1029,7 @@ connman_bool_t connman_network_get_associating(struct connman_network *network)
  */
 int __connman_network_connect(struct connman_network *network)
 {
+	struct connman_service *service;
 	int err;
 
 	DBG("network %p", network);
@@ -924,10 +1046,15 @@ int __connman_network_connect(struct connman_network *network)
 	if (network->driver->connect == NULL)
 		return -ENOSYS;
 
+	if (network->device == NULL)
+		return -ENODEV;
+
 	__connman_device_disconnect(network->device);
 
 	network->connecting = TRUE;
 
+	service = __connman_service_lookup_from_network(network);
+
 	err = network->driver->connect(network);
 	if (err < 0) {
 		if (err == -EINPROGRESS)
@@ -979,6 +1106,144 @@ int __connman_network_disconnect(struct connman_network *network)
 	return err;
 }
 
+static int dhcp_start(struct connman_network *network)
+{
+	struct connman_element *element;
+	int error;
+
+	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
+		return -EINVAL;
+
+	element = connman_element_create(NULL);
+	if (element == NULL)
+		return -ENOMEM;
+
+	element->type  = CONNMAN_ELEMENT_TYPE_DHCP;
+	element->index = network->element.index;
+
+	error = connman_element_register(element, &network->element);
+	if (error < 0) {
+		connman_element_unref(element);
+		return error;
+	}
+
+	return 0;
+}
+
+static int dhcp_stop(struct connman_network *network)
+{
+	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
+		return -EINVAL;
+
+	connman_element_unregister_children_type(&network->element,
+					CONNMAN_ELEMENT_TYPE_CONNECTION);
+	connman_element_unregister_children_type(&network->element,
+						CONNMAN_ELEMENT_TYPE_IPV4);
+	connman_element_unregister_children_type(&network->element,
+						CONNMAN_ELEMENT_TYPE_DHCP);
+
+	return 0;
+}
+
+static int manual_ipv4_set(struct connman_network *network,
+				struct connman_ipconfig *ipconfig)
+{
+	struct connman_service *service;
+	int err;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return -EINVAL;
+
+	err = __connman_ipconfig_set_address(ipconfig);
+	if (err < 0) {
+		connman_network_set_error(network,
+			CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL);
+		return err;
+	}
+
+	__connman_ipconfig_set_gateway(ipconfig, &network->element);
+
+	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY);
+
+	return 0;
+}
+
+int __connman_network_clear_ipconfig(struct connman_network *network,
+					struct connman_ipconfig *ipconfig)
+{
+	struct connman_service *service;
+	enum connman_ipconfig_method method;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return -EINVAL;
+
+	method = __connman_ipconfig_get_method(ipconfig);
+
+	switch (method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+		return -EINVAL;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		connman_element_unregister_children_type(&network->element,
+					CONNMAN_ELEMENT_TYPE_CONNECTION);
+		__connman_ipconfig_clear_address(ipconfig);
+		break;
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		dhcp_stop(network);
+		break;
+	}
+
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION);
+
+	return 0;
+}
+
+int __connman_network_set_ipconfig(struct connman_network *network,
+					struct connman_ipconfig *ipconfig)
+{
+	struct connman_ipconfig *ipv6config;
+	enum connman_ipconfig_method method;
+	int ret;
+
+	ipv6config = connman_ipconfig_get_ipv6config(ipconfig);
+	method = __connman_ipconfig_get_method(ipv6config);
+	switch (method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		ret = manual_ipv6_set(network, ipv6config);
+		if (ret != 0) {
+			connman_network_set_error(network,
+				CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
+			return FALSE;
+		}
+		break;
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		break;
+	}
+
+	method = __connman_ipconfig_get_method(ipconfig);
+
+	switch (method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+		return -EINVAL;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		return manual_ipv4_set(network, ipconfig);
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		return dhcp_start(network);
+	}
+
+	return 0;
+}
+
 /**
  * connman_network_set_address:
  * @network: network structure
@@ -1176,6 +1441,34 @@ const char *connman_network_get_string(struct connman_network *network,
 	return connman_element_get_string(&network->element, key);
 }
 
+/**
+ * connman_network_set_passphrase:
+ * @network: network structure
+ * @passphrase: network passphrase
+ *
+ * Set network passphrase.
+ * If the network is linked to a service, the latter gets his passphrase
+ * set as well.
+ */
+int connman_network_set_passphrase(struct connman_network *network,
+						const char* passphrase)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL) {
+		connman_network_set_string(network, "WiFi.Passphrase",
+						passphrase);
+
+		return 0;
+	}
+
+	__connman_service_set_passphrase(service, passphrase);
+
+	return 0;
+}
+
+
 /**
  * connman_network_set_bool:
  * @network: network structure
@@ -1383,6 +1676,27 @@ void connman_network_set_data(struct connman_network *network, void *data)
 	network->driver_data = data;
 }
 
+void connman_network_update(struct connman_network *network)
+{
+	switch (network->type) {
+	case CONNMAN_NETWORK_TYPE_UNKNOWN:
+	case CONNMAN_NETWORK_TYPE_VENDOR:
+		return;
+	case CONNMAN_NETWORK_TYPE_ETHERNET:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_CELLULAR:
+	case CONNMAN_NETWORK_TYPE_WIFI:
+	case CONNMAN_NETWORK_TYPE_WIMAX:
+		break;
+	}
+
+	if (network->group != NULL)
+		__connman_service_update_from_network(network);
+
+	return;
+}
+
 static gboolean match_driver(struct connman_network *network,
 					struct connman_network_driver *driver)
 {
@@ -1469,7 +1783,8 @@ static void network_remove(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		if (network->group != NULL) {
-			__connman_profile_remove_network(network);
+			if (network->registered)
+				__connman_profile_remove_network(network);
 
 			g_free(network->group);
 			network->group = NULL;
